#!/bin/bash
#
# bash-common-api:bash-common-api.sh
#
if [ "$( type -t echofatal )" != 'function' ] ; then
  # shellcheck disable=SC1090
  source "$( dirname "${BASH_SOURCE[0]}" )/message-api.sh" || exit 1
fi

function is_variable_define_and_not_empty_safe {
  local -r var_name="$1"

  (
    set +u

    local var_value="${!var_name}"

    if [ -n "${var_value}" ] ; then
      return 0
    else
      return 1
    fi
  )
}

function is_positive_integer {
  if [[ "$1" =~ ^[0-9]+$ ]] ; then
    return 0
  else
    return 1
  fi
}

function exit_if_variable_not_define_or_empty {
  local -r var_name="$1"

  if ! is_variable_define_and_not_empty_safe "${var_name}" ; then
    echofatal "Missing ${var_name} - ${var_name} must be define and not empty."
    exit 1
  fi
}

function exit_if_variable_not_define_and_or_empty { # DEPRECATED - DISABLED 2021-02-06
  echowarn 'exit_if_variable_not_define_and_or_empty() is deprecated - use exit_if_variable_not_define_or_empty() instead' >&2
  exit 100
}

function _print_variable_unsafe {
  local -r var_name="$1"

  (
    set -u

    echo "${!var_name}"
  )
}

function get_variable_safe {
  local -r var_name="$1"

  if _print_variable_unsafe  "${var_name}" 2>/dev/null ; then
    return 0
  else
    return 1
  fi
}

function is_variable_define_safe {
  local -r var_name="$1"

  if _print_variable_unsafe  "${var_name}" 1>/dev/null 2>&1 ; then
    return 0
  else
    return 1
  fi
}

function exit_if_variable_not_define {
  local -r var_name="$1"

  if ! is_variable_define_safe "${var_name}" ; then
    echofatal "Missing ${var_name} - ${var_name} must be define."
    exit 1
  fi
}

function exit_if_directory_not_exists {
  local -r dir="$1"

  if [ ! -d "${dir}" ] ; then
    echofatal "Directory '${dir}' is required."
    exit 1
  fi
}

function exit_if_file_not_exists {
  local -r file="$1"

  if [ ! -f "${file}" ] ; then
    echofatal "File '${file}' is required."
    exit 1
  fi
}

function exit_if_command_is_missing {
  local command=

  while [ "$#" -gt 0 ] ; do
    command="$1"
    shift

    if ! command -v "${command}" >/dev/null ; then
      echofatal "${command} is missing."
      exit 1
    fi
  done
}

function is_boolean {
  local -r value="$1"

  case "${value}" in
    'true'|'false') return 0 ;;
    *) return 1 ;;
  esac
}

# P: get_required_next_param
# Usage:
#   varname="$( P "$@" )" && shift
# or
#   if ! param_xxx="$( P "$@" )" ; then
#     echofatal "Value required for '${param}'"
#     usage >&2
#     exit 1
#   fi
#   shift
#   ...
function P {
  if [ "$#" -gt 0 ] && ! [[ "$1" =~ ^--.* ]] ; then
    echo "$1"
    return 0
  fi
  return 1
}

function convert_url_to_safe_filename {
  local -r url="$1"

  # shellcheck disable=SC2001
  sed -e 's/[^A-Za-z0-9._-]/_/g' <<<"${url}"
}

#
# trim each lines of stdin
#
function trim {
  sed 's/^[[:space:]]*//' | sed 's/*[[:space:]]$//'
}

