#!/bin/bash
#
# bash-common-api:json-api.sh
#
if [ "$( type -t echoinfo )" != 'function' ] ; then
  # shellcheck disable=SC1090
  source "$( dirname "${BASH_SOURCE[0]}" )/message-api.sh" || exit 1
fi

function load_vars_from_json {
  local -r json_file="$1"
  local -r json_path="$2"

  if [ ! -f "${json_file}" ] ; then
    echofatal "Can't find: '${json_file}'."
    return 1
  fi

  local keys=
  keys="$( jq -re "${json_path} | keys[]" < "${json_file}" )" || return $?

  local key=
  local value=
  while read -r key ; do
    if [ -n "${key}" ] ; then
      echoinfo "Set '${key}'"
      value="$( jq -re "${json_path}.${key}" < "${json_file}" )" || return $?

      declare -gr "${key}"="${value}"
      # shellcheck disable=SC2163
      export "${key}" # required by envsubst
    fi
  done <<<"${keys}"
}

function get_expanded_value_from_json {
  local -r json_str="$1"
  local -r key="$2"

  local value=
  value="$( jq -re "${key}" <<<"${json_str}" )" || return $?

  envsubst <<<"${value}" || return $?
}

function is_null_or_empty {
  local -r value="$1"

  if [ "${value}" = 'null' ] ; then
    return 0
  elif [ -z "${value}" ] ; then
    return 0
  fi

  return 1
}
