#!/bin/bash
#
# freebox-tools:bin/freebox-audit-api.sh
#
# Try to get more human readable freebox configuration view
#

function display_version {
  cat <<EOF
$( realpath "${BASH_SOURCE[0]}" )
APP_VERSION : ${_APP_VERSION_}
APP_ID      : ${_APP_ID_}
APP_NAME:   : ${_APP_NAME_}
EOF
}


function show_api_json {
  local answer=
  answer="$( freebox_api_version_json )" || exit $?

  jq '{
  box_model_name: .box_model_name,
  api_base_url: .api_base_url,
  device_name: .device_name,
  box_model: .box_model,
  device_type: .device_type,
  https_available: .https_available,
  https_port: .https_port,
  uid: .uid,
  api_version: .api_version,
  api_domain: .api_domain
  }' <<<"${answer}" || exit 1
}

function show_api_text {
  local answer=
  answer="$( show_api_json )" || exit $?

  cat <<EOF
Nom du modèle           : $( jq -r '.box_model_name' <<<"${answer}" )
Préfix de l'URI de l'API: $( jq -r '.api_base_url' <<<"${answer}" )
Nom du périphérique     : $( jq -r '.device_name' <<<"${answer}" )
Modèle                  : $( jq -r '.box_model' <<<"${answer}" )
Type                    : $( jq -r '.device_type' <<<"${answer}" )

Support https           : $( jq -r '.https_available' <<<"${answer}" )
Port https publique     : $( jq -r '.https_port' <<<"${answer}" )

UID du périphérique     : $( jq -r '.uid' <<<"${answer}" )
Version de l'API        : $( jq -r '.api_version' <<<"${answer}" )
Domaine de l'API        : $( jq -r '.api_domain' <<<"${answer}" )

EOF

}

function show_connection_json {
  local answer=
  answer="$( freebox_api_get '/connection' )" || exit $?

  jq '{
  bandwidth_down: .result.bandwidth_down,
  bandwidth_up: .result.bandwidth_up,
  ipv4: .result.ipv4,
  ipv6: .result.ipv6,
  state: .result.state,
  type: .result.type
  }' <<<"${answer}" || exit $?
}

function show_connection_text {
  local answer=
  answer="$( show_connection_json )" || exit $?

  cat <<EOF
État                                    : $( jq -r '.state' <<<"${answer}" )
Type                                    : $( jq -r '.type' <<<"${answer}" )
Adresse publique IPv4                   : $( jq -r '.ipv4' <<<"${answer}" )
Adresse publique IPv6                   : $( jq -r '.ipv6' <<<"${answer}" )
Bande passante montante      (upload)   : $( jq -r '.bandwidth_up' <<<"${answer}" ) bit/s
Bande passante descendante (download)   : $( jq -r '.bandwidth_down' <<<"${answer}" ) bit/s

EOF
}
