#!/bin/bash
#
# freebox-tools:bin/freebox-tools-api.sh
#
#
if [ "$( type -t P )" != 'function' ] ; then
  echo 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function P not found.' >&2
  exit 1
fi

declare -gr __JQ_NORMALIZE__="${PROJECT_HOME}/bin/normalize.jq"

declare -g param_active=
declare -g param_family=

#
# Freebox Tools
#
function freebox_reboot {
  freebox_api_get '/system/reboot' '{}' | jq .
}

function freebox_reset_all_plugs_raw {
  local id=
  local first='true'
  local result=
  local success=

  echo -n '['
  while read -r id ; do
    echoinfo "Rédémarrage du plug: ${id}" >&2
    if [ "${first}" = 'true' ] ; then
      first='false'
    else
      echo -n ','
    fi
    result="$( freebox_api_post "/freeplug/${id}/reset/" )" || return $?
    success="$( jq -r '.success' <<<"${result}" )" || return $?
    echo -n "{ \"id\": \"${id}\", \"success\": ${success} }"
    sleep 5
  done < <( freebox_api_get '/freeplug' | jq -r '.result // [] | .[].members[].id' )
  echo -n ']'
}

function freebox_reset_all_plugs {
  freebox_reset_all_plugs_raw | jq .
}

function __hosts_filter_active__ {
  case "${param_active}" in
    '')
      jq '.'
      ;;

    'true'|'false')
      jq "{ success: .success, result: [ .result // [] | .[] |
        select( .l3connectivities // [] | .[] | .active==${param_active} )
        ] }"
      ;;

    *)
      echofatal "--active must be empty, true or false. Found: '${param_active}'"
      return 100
      ;;
  esac
}

function __hosts_filter_family__ {
  case "${param_family}" in
    '')
      jq '.'
      ;;

    'ipv4'|'ipv6')
      jq "{ success: .success, result: [ .result // [] | .[] |
         del( select( .l3connectivities // [] | .[] | .af!=\"${param_family}\" ) ) // empty
         ] }"
      ;;

    *)
      echofatal "--family must be empty, ipv4 or ipv6. Found: '${param_family}'"
      return 100
      ;;
  esac
}

function jq_normalize {
  jq -S -f "${__JQ_NORMALIZE__}"
}

function __hosts_filters__ {
  __hosts_filter_family__ | __hosts_filter_active__ | jq_normalize
}

function get_hosts_json {
  freebox_api_get '/lan/browser/pub' | __hosts_filters__ | jq '. | .result |= ( sort_by( .id ) )'
}

#
# MAC Addresses
#

function freebox_show_mac_addresses_json {
  get_hosts_json |
    jq '[ .result[].l2ident.id ]' |
    jq_normalize || exit $?
}

function freebox_show_mac_addresses_text {
  freebox_show_mac_addresses_json | jq -r '.[]' || exit 1
}

#
# display commands: formatted lines
#

#eclare -r DEFAULT_PLACE_HOLDER='-----------------------------'
declare -r DEFAULT_WHITE_PLACE_HOLDER='                                                                '
declare -r DEFAULT_SEPARATOR='\t'

#eclare -g PLACE_HOLDER="${DEFAULT_PLACE_HOLDER}"
declare -g WHITE_PLACE_HOLDER="${DEFAULT_WHITE_PLACE_HOLDER}"
declare -g SEPARATOR="${DEFAULT_SEPARATOR}"
declare -g END_SEPARATOR=''

function _freebox_display_hosts_json_raw_ {
  local first='true'
  local host_json=

  echo '['
  while read -r host_json ; do
    if [ "${first}" = 'true' ] ; then
      first='false'
    else
      echo ','
    fi

    jq '{
  names: .names,
  vendor_name: .vendor_name,
  l2ident: .l2ident,
  l3connectivities: .l3connectivities,
  host_type: .host_type
  }' <<<"${host_json}" || exit 1

  done < <( get_hosts_json | jq -c ".result[]" )
  echo ']'
}

function freebox_display_hosts_json {
  _freebox_display_hosts_json_raw_ | jq_normalize
}

function freebox_display_hosts_text {
  local host_answer=
  local host_ipv4=
  local host_ipv6=
  local host_mac=
  local host_name=
  local host_type=
  local host_vendor=

  while read -r host_answer; do
    host_ipv4="$( jq -r '[ .l3connectivities // [] | .[] | select( .af == "ipv4") ][0].addr' <<<"${host_answer}" )${WHITE_PLACE_HOLDER}" || exit 1
    host_ipv6="$( jq -r '[ .l3connectivities // [] | .[] | select( .af == "ipv6") ][0].addr' <<<"${host_answer}" )${WHITE_PLACE_HOLDER}" || exit 1
    host_mac="$( jq -r ".l2ident.id" <<<"${host_answer}" )" || exit 1
    host_name="$( jq -r ".names[0].name" <<<"${host_answer}" )" || exit 1
    host_type="$( jq -r ".host_type" <<<"${host_answer}" )" || exit 1
    host_vendor="$( jq -r ".vendor_name" <<<"${host_answer}" )" || exit 1

    host_type="${host_type}${WHITE_PLACE_HOLDER}"
    host_vendor="${host_vendor}${WHITE_PLACE_HOLDER}${WHITE_PLACE_HOLDER}"
    host_name="${host_name}${WHITE_PLACE_HOLDER}${WHITE_PLACE_HOLDER}"

    echo -en "${host_ipv4:0:15}${SEPARATOR}"
    echo -en "${host_ipv6:0:26}${SEPARATOR}"
    echo -en "${host_mac,,}${SEPARATOR}"
    echo -en "${host_type:0:20}${SEPARATOR}"
    echo -en "${host_vendor:0:64}${SEPARATOR}"
    echo -en "${host_name:0:64}"
    echo -e  "${END_SEPARATOR}"
  done < <( freebox_display_hosts_json | jq -c '.[]' )
}
