#!/bin/bash -e
#
# freebox-tools:bin/freebox.sh
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -r PROJECT_HOME

# shellcheck disable=SC1090
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit $?
# shellcheck disable=SC1090
source "${PROJECT_HOME}/bin/freebox-tools-api.sh" || exit $?
# shellcheck disable=SC1090
source "${PROJECT_HOME}/bin/freebox-bash-api.sh" || exit $?
# shellcheck disable=SC1090
source "${PROJECT_HOME}/bin/freebox-config-api.sh" || exit $?
# shellcheck disable=SC1090
source "${PROJECT_HOME}/bin/freebox-parental-api.sh" || exit $?
# shellcheck disable=SC1090
source "${PROJECT_HOME}/bin/freebox-audit-api.sh" || exit $?

function usage {
  cat <<EOF
Usage:
  $( basename "${BASH_SOURCE[0]}" ) --help

  $( basename "${BASH_SOURCE[0]}" ) --init-token

  $( basename "${BASH_SOURCE[0]}" ) --api
  $( basename "${BASH_SOURCE[0]}" ) --api-text
  $( basename "${BASH_SOURCE[0]}" ) --connection
  $( basename "${BASH_SOURCE[0]}" ) --connection-text
  $( basename "${BASH_SOURCE[0]}" ) --display-hosts
  $( basename "${BASH_SOURCE[0]}" ) --display-hosts-text [--separator SEPARATOR] [--separator LASTSEP]
  $( basename "${BASH_SOURCE[0]}" ) --json URI # Identique à GET
  $( basename "${BASH_SOURCE[0]}" ) --parental-config
  $( basename "${BASH_SOURCE[0]}" ) --parental-filter ID
  $( basename "${BASH_SOURCE[0]}" ) --parental-filters
  $( basename "${BASH_SOURCE[0]}" ) --parental-filters-id
  $( basename "${BASH_SOURCE[0]}" ) --plug
  $( basename "${BASH_SOURCE[0]}" ) --restart
  $( basename "${BASH_SOURCE[0]}" ) --show-mac-address
  $( basename "${BASH_SOURCE[0]}" ) --version

  $( basename "${BASH_SOURCE[0]}" ) get URI
  $( basename "${BASH_SOURCE[0]}" ) post URI DATA
  $( basename "${BASH_SOURCE[0]}" ) put URI DATA

Boite à outils pour votre Freebox. Notez que certaines fonctionnalités demandent des droits
supplémentaires qu’il faut ajouter depuis l’interface de votre Freebox.

Actions:
  -h,   --help          Affiche ce message d’aide et quitte.

  -I,   --init-token    Configuration: Récupère un nouveau token pour l’application
                        et le sauvegarde.

        --api           Affiche les informations disponibles sur l’API de votre
                        Freebox au format JSON.
        --api-text      Affiche les informations disponibles sur l’API de votre
                        Freebox au format TEXTE.

        --calls,
        --call-log      Retourne la liste des appels au format JSON.
        --call-numbers-text
                        Affiches la liste des numéros de téléphones ayant appelés au
                        format TEXTE (un numéro par ligne).

  -C,   --connection    Affiche certaines informations de connexion de votre Freebox
                        au format JSON.
        --connection-text
                        Affiche certaines informations de connexion de votre Freebox.
                        au format TEXTE.

        --display-hosts Affiche la liste des machines connues au format JSON.
        --display-hosts-text
                        Affiche la liste des machines connues au format TEXTE:
                            Adresse IPv4
                            Adresse IPv6
                            Adresse MAC
                            Type de la machine
                            Constructeur de la machine
                            Nom de la machine

        --hosts         Affiche la liste des machines détaillée connues
                        (raccourci pour GET /lan/browser/pub)

        --parental-config
                        Configuration du contrôle parental au format JSON

        --parental-filter ID
                        Configuration du filtre ID du contrôle parental au format JSON

                        Par exemple:

                        ${BASH_SOURCE[0]} --parental-filter 3

        --parental-filters
                        Configuration de tous les filtres du contrôle parental au format
                        JSON

        --parental-filters-id
                        Affiche le nom et l’id des filtres du contrôle parental au format
                        JSON

  -P,   --plug          Liste de vos boîtiers CPL (freeplugs et autre).
        --plug-reset ID Redémarrage du boîtier CPL donné
        --plug-reset-all
                        Redémarrage de tous les boîtiers CPL trouvés.

        --profile [ID]  Sans paramètre affiche la liste de tous les profils, si ID est
                        un nombre affiche le profil correspondant à l'ID.
                        [Nécessite l'API V8]

        --profile-network [ID]
                        Sans paramètre affiche toutes les configurations, si ID est
                        un nombre affiche le profil réseaux correspondant à l'ID.

  -R,   --restart       Demande un redémarrage de la Freebox.

        --show-mac-address
                        Retourne la liste des adresses MAC connues par votre
                        Freebox au format JSON.
        --show-mac-address-text
                        Retourne la liste des adresses MAC connues par votre
                        Freebox au format TEXTE.

        --version       Affiche la version et quitte.

        get URI,
        --api-get URI,
        --json URI      Lance une requête HTTP/GET sur la box et affiche le
                        résultat brut au format JSON. Les éventuels paramètres
                        doivent être correctement encodés.

                        Quelques exemples:

                        Caractéristique de votre ligne
                        ${BASH_SOURCE[0]} get /connection/xdsl

                        Votre configuration DHCP
                        ${BASH_SOURCE[0]} get /dhcp/config

                        Vos règles de délégations (forwarding)
                        ${BASH_SOURCE[0]} get /fw/redir/

                        La configuration du contrôle parental
                        ${BASH_SOURCE[0]} get /parental/config/

        post URI DATA,
        --api-post URI DATA
                        Lance une requête HTTP/POST sur la box avec la chaîne DATA qui
                        doit être un JSON bien formé et affiche le
                        résultat brut au format JSON.

        put URI DATA,
        --api-put URI DATA
                        Lance une requête HTTP/PUT sur la box avec la chaîne DATA qui
                        doit être un JSON bien formé et affiche le
                        résultat brut au format JSON.

                        Par exemple:

                        ${BASH_SOURCE[0]} put '/parental/filter/2' '{"forced_mode": "allowed", "forced": true}'
                        ${BASH_SOURCE[0]} put '/parental/filter/2' '{"forced_mode": "denied", "forced": true}'
                        ${BASH_SOURCE[0]} put '/parental/filter/2' '{"forced": false}'

                        N'est plus supporté depuis la version 8 de l’API (non documenté)
                        ${BASH_SOURCE[0]} put '/parental/config' '{"default_filter_mode": "denied"}


Options:


  -A BOOL,      --active BOOL
                        N'affiche que les machines étant actuellement connectée.

  -F FAMILY,    -f FAMILY,      --family FAMILY
                        Les interfaces de la famille donnée. FAMILY peut prendre l'une des
                        valeurs suivantes: ipv4 ou ipv6

  -S SEPARATOR, --separator SEPARATOR
                        Défini le séparateur de colonne (défaut: ${SEPARATOR})

  -L LASTSEP,   --last-separator LASTSEP
                        Défini le séparateur de fin de ligne (défaut: ${END_SEPARATOR})

  -k,   --insecure      Autorise les connexions non certifiées lors de l’utilisation de SSL

        --http, --allow-http
                        Autorise l’utilisation du protocole http (non sécurisé)

EOF
}

function is_required {
  local -r value="$1"
  local -r message="$2"

   if [ -z "${value}" ] ; then
    echofatal "${message}" >&2
    echo >&2
    usage >&2
    exit 1
  fi
}

function main {
  local action=
  local id=
  local uri=
  local data=
  local param=

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help')
        usage
        exit 0
        ;;

      '--version')
        display_version
        exit 0
        ;;

      '-I'|'--init-token')
        action='--init-token'
        ;;

      '-R'|'--restart')
        action='--restart'
        ;;

      'get'|'--api-get'|'--json')
        action='GET'
        uri="$( P "$@" )" && shift
        ;;

      'post'|'--api-post')
        action='POST'
        uri="$( P "$@" )" && shift
        ;;

      'put'|'--api-put')
        action='PUT'
        uri="$( P "$@" )" && shift
        data="$( P "$@" )" && shift
        ;;

      '-C'|'--connection')
        action='--connection'
        ;;

      '--calls'|'--call-log')
        action='--calls'
        ;;

      '-P'|'--plug')
        action='--plug'
        ;;

      '--parental-filter'|\
      '--plug-reset')
        action="${param}"
        id="$( P "$@" )" && shift
        ;;

      '--network-control'|\
      '--profile'|\
      '--profile-network')
        action="${param}"
        if is_positive_integer "${1:-}" ; then
          id="$( P "$@" )" && shift
        else
          id='0'
        fi
        ;;

      '--api'|'--api-text'|\
      '--call-numbers-text'|\
      '--connection-text'|\
      '--display-hosts'|\
      '--display-hosts-text'|\
      '--hosts'|\
      '--parental-config'|\
      '--parental-filters'|\
      '--parental-filters-id'|\
      '--plug-reset-all'|\
      '--show-mac-address'|\
      '--show-mac-address-text')
        action="${param}"
        ;;

      '-A'|'--active')
        # shellcheck disable=SC2034
        param_active="$( P "$@" )" && shift
        ;;

      '-F'|'-f'|'--family')
        # shellcheck disable=SC2034
        param_family="$( P "$@" )" && shift
        ;;

      '-S'|'--separator')
        SEPARATOR="$( P "$@" )" && shift
        ;;

      '-L'|'--last-separator')
        END_SEPARATOR="$( P "$@" )" && shift
        ;;

      '-k'|'--insecure'         ) set_insecure ;;
      '--http'|'--allow-http'   ) set_allow_http ;;

      *)
        # unknown option
        echofatal "Paramètre inconnu: '${param}'."
        usage >&2
        exit 1
        ;;
    esac
  done

  case "${action}" in
    '')
      echo 'Paramètre manquant.' >&2
      usage >&2
      exit 1
      ;;

    '--init-token'          ) freebox_config_init_token || return $? ;;

    '--api'                 ) freebox_connect && show_api_json ;;
    '--api-text'            ) freebox_connect && show_api_text ;;
    '--call-numbers-text'   ) freebox_connect && freebox_api_get_with_result_if_success '/call/log/' | jq -r '.[].number' ;;
    '--calls'               ) freebox_connect && freebox_api_get_with_result_if_success '/call/log/' | jq . ;;
    '--connection'          ) freebox_connect && show_connection_json ;;
    '--connection-text'     ) freebox_connect && show_connection_text ;;
    '--display-hosts'       ) freebox_connect && freebox_display_hosts_json ;;
    '--display-hosts-text'  ) freebox_connect && freebox_display_hosts_text ;;
    '--hosts'               ) freebox_connect && get_hosts_json ;;
    '--parental-config'     ) freebox_connect && freebox_api_get '/parental/config/' | jq_normalize ;;
    '--parental-filters'    ) freebox_connect && freebox_getparental_filters_json ;;
    '--plug'                ) freebox_connect && freebox_api_get '/freeplug' | jq_normalize ;;
    '--plug-reset-all'      ) freebox_connect && freebox_reset_all_plugs ;;
    '--restart'             ) freebox_connect && freebox_reboot ;;
    '--show-mac-address'        ) freebox_connect && freebox_show_mac_addresses_json ;;
    '--show-mac-address-text'   ) freebox_connect && freebox_show_mac_addresses_text ;;
    '--network-control'         ) freebox_connect && freebox_network_control_json "${id}" ;;

    '--parental-filter')
      is_required "${id}" "L'action --parental-filter attend un id (ex: --parental-filter '1')"
      freebox_connect && freebox_api_get "/parental/filter/${id}" | jq_normalize
      ;;

    '--parental-filters-id')
      freebox_connect && freebox_getparental_filters_json |
        jq '[ .[] | { id:.id, desc:.desc } ]' | jq_normalize
      ;;

    '--plug-reset')
      is_required "${id}" 'Le paramètre --plug-reset attend un id (ex: --plug-reset "F4:CA:E5:1D:46:AE")'
      freebox_connect
      freebox_api_post "/freeplug/${id}/reset/" | jq .
      ;;

    '--profile')
        freebox_connect
        if [ "${id}" -eq 0 ] ; then
          freebox_api_get '/profile/' | jq .
        else
          freebox_api_get "/profile/${id}" | jq .
        fi
        ;;

    '--profile-network')
        freebox_connect
        if [ "${id}" -eq 0 ] ; then
          freebox_api_get '/network_control' | jq .
        else
          freebox_api_get "/network_control/${id}" | jq .
        fi
        ;;

    'GET')
      is_required "${uri}" "L'action GET attend un suffixe d'API (ex: GET '/lan/browser/interfaces/')"
      freebox_connect
      freebox_api_get "${uri}" | jq .
      ;;

    'POST')
      is_required "${uri}" "L'action POST attend un suffixe d'API (ex: POST '/lan/browser/interfaces/')"
      freebox_connect
      freebox_api_post "${uri}" | jq .
      ;;

    'PUT')
      is_required "${uri}" "L'action PUT attend 2 paramètres: le suffix d'API et les données au format JSON')"
      is_required "${data}" "L'action PUT attend 2 paramètres: le suffix d'API et les données au format JSON')"
      freebox_connect
      freebox_api_put "${uri}" "${data}" | jq .
      ;;

    *)
      echoerror "(BUG) Action non gérée: '${action}'." >&2
      usage >&2
      exit 1
      ;;
  esac
}

main "$@" || exit $?
